//
//  SPI.cpp
//  Cyclop_OSD_CPP
//
//  Created by Dominik Loher on 13.09.16.
//  Copyright © 2016 Dominik Loher. All rights reserved.
//

#include "SPI.h"

#include <avr/io.h>
#include <avr/interrupt.h>
#define RESET 0	//PB0
#define SS 2	//PB2
#define MOSI 3	//PB3
#define MISO 4	//PB4
#define SCK 5	//PB5

void spi_init()
// Initialize pins for spi communication
{
//    DDRB &= ~((1<<MOSI)|(1<<MISO)|(1<<SS)|(1<<SCK));
    // Define the following pins as output
    DDRB |= ((1<<MOSI)|(1<<SS)|(1<<SCK)) | (1<<RESET);
    SPCR = ((1<<SPE)|               // SPI Enable
            (0<<SPIE)|              // SPI Interupt Enable
            (0<<DORD)|              // Data Order (0:MSB first / 1:LSB first)
            (1<<MSTR)|              // Master/Slave select
            (0<<SPR1)|(0<<SPR0)|    // SPI Clock Rate
            (0<<CPOL)|              // Clock Polarity (0:SCK low / 1:SCK hi when idle)
            (0<<CPHA));             // Clock Phase (0:leading / 1:trailing edge sampling)
    
    SPSR = (1<<SPI2X);              // Double Clock Rate
    
    PORTB |= (1<<PB2);              //set SS to hight = deselected
}

void spi_selectSlave(bool select){
    if( select ){
        PORTB &= ~(1<<PB2);          //set SS to low , selected
    }else{
        PORTB |= (1<<PB2);
    }
}

void spi_transfer_sync (uint8_t * dataout, uint8_t * datain, uint8_t len)
// Shift full array through target device
{
    uint8_t i;
    for (i = 0; i < len; i++) {
        SPDR = dataout[i];
        while((SPSR & (1<<SPIF))==0);
        datain[i] = SPDR;
    }
}

void spi_transmit_sync (uint8_t * dataout, uint8_t len)
// Shift full array to target device without receiving any byte
{
    uint8_t i;
    for (i = 0; i < len; i++) {
        SPDR = dataout[i];
        while((SPSR & (1<<SPIF))==0);
    }
}

uint8_t spi_fast_shift (uint8_t data)
// Clocks only one byte to target device and returns the received one
{
    SPDR = data;
    while((SPSR & (1<<SPIF))==0);
    return SPDR;
}